SystemOrganization addCategory: #ScratchSoundEditor!

RectangleMorph subclass: #ScratchGraphMorph
	instanceVariableNames: 'data dataColor playCursorColor playing cursorColor cursorColorAtZeroCrossings startIndex minVal maxVal selection scale cachedForm hasChanged ruler viewer trueCalls falseCalls negVals posVals editor minScale'
	classVariableNames: 'MinScale'
	poolDictionaries: ''
	category: 'ScratchSoundEditor'!

!ScratchGraphMorph commentStamp: 'tpr 12/6/2013 18:23' prior: 0!
A SCratchGraphMorph is an apparently unused class.

I display a graph of numbers, normalized so the full range of values just fits my height. I support a movable cursor that can be dragged with the mouse.

Implementation notes: Some operations on me may be done at sound sampling rates (e.g. 11-44 thousand times/second). To allow such high bandwidth application, certain operations that change my appearance do not immediately report a damage rectangle. Instead, a flag is set indicating that my display needs to refreshed and a step method reports the damage rectangle if that flag is set. Also, I cache a bitmap of my graph to allow the cursor to be moved without redrawing the graph.

All indices, like startIndex, cursor, etc are in terms of the graph data.

IMPORTANT!!  The current implementation cannot stand alone, it needs to be a submorph of SoundEditor.
!

!ScratchGraphMorph class methodsFor: 'instance creation' stamp: 'LY 7/30/2003 11:52'!
MinScale

	^MinScale! !

!ScratchGraphMorph class methodsFor: 'instance creation' stamp: 'jm 9/5/2006 18:27'!
MinScale: aNumber

	"aNumber should be equal to: viewer extent x/ graph data size"
	MinScale := aNumber asFloat.
! !

!ScratchGraphMorph class methodsFor: 'instance creation' stamp: 'jm 7/12/2003 17:23'!
openOn: dataCollection
	"Open a new GraphMorph on the given sequencable collection of data."

	^ (self new data: dataCollection) openInWorld
! !

!ScratchGraphMorph methodsFor: 'menu' stamp: 'jm 7/12/2003 17:23'!
addCustomMenuItems: aCustomMenu hand: aHandMorph

	super addCustomMenuItems: aCustomMenu hand: aHandMorph.
	aCustomMenu add: 'open wave editor' action: #openWaveEditor.
	aCustomMenu add: 'read file' action: #readDataFromFile.
! !

!ScratchGraphMorph methodsFor: 'menu' stamp: ''!
adjustSelection
	"Adjust the selection, if any, to the current cursor position. Do nothing if there is no selection."

	editor selectionNil ifTrue: [^ self].
	editor selection: editor cursor scd: editor startSelection.

! !

!ScratchGraphMorph methodsFor: 'commands' stamp: 'jm 7/12/2003 17:23'!
appendValue: aPointOrNumber

	| newVal |
	(data isKindOf: OrderedCollection) ifFalse: [data := data asOrderedCollection].
	newVal := self asNumber: aPointOrNumber.
	data addLast: newVal.
	newVal < minVal ifTrue: [minVal := newVal].
	newVal > maxVal ifTrue: [maxVal := newVal].
	self cursor: data size.
	self flushCachedForm.
! !

!ScratchGraphMorph methodsFor: 'private' stamp: ''!
calculateDataArray

	| currIndex neg pos |

	negVals := OrderedCollection new.
	posVals := OrderedCollection new.

	data isEmpty ifTrue: [^ self].
	currIndex :=  neg := pos := 0.

	(1 to: data size) do: [ :i |
		((i * scale) truncated > currIndex)
			ifTrue: [
			
					currIndex := (i*scale) truncated.		
					neg := neg min: (data at: i).
					pos := pos max: (data at: i).
			
					posVals add: pos.
					negVals	add: neg.
		
					pos := neg := 0.]
			ifFalse: [
						neg := neg min: (data at: i).
						pos := pos max: (data at: i).].].! !

!ScratchGraphMorph methodsFor: 'commands' stamp: ''!
centerCursor
	"Scroll so that the cursor is as close as possible to the center of my window."

	| w |

	w := self width - (2 * borderWidth).
	self startIndex: ((editor cursor - (w // (scale*2))) max: 1).
! !

!ScratchGraphMorph methodsFor: 'commands' stamp: 'jm 7/12/2003 17:23'!
clear

	self startIndex: 1.
	self cursor: 1.
	self data: OrderedCollection new.
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'jm 7/12/2003 17:23'!
color: aColor

	super color: aColor.
	self flushCachedForm.
! !

!ScratchGraphMorph methodsFor: 'viewing' stamp: 'tfel 8/3/2016 14:09'!
computeSlider
	
	editor slider changed.! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: ''!
cursor

	^ editor cursor
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: ''!
cursorAtEnd

	^editor cursor truncated >= data size
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'jm 7/12/2003 17:23'!
cursorColor

	^ cursorColor
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'jm 7/12/2003 17:23'!
cursorColorAtZeroCrossing

	^ cursorColorAtZeroCrossings
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'jm 7/12/2003 17:23'!
cursorColorAtZeroCrossings: aColor

	cursorColorAtZeroCrossings := aColor.
	self flushCachedForm.
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'jm 7/12/2003 17:23'!
cursorColor: aColor

	cursorColor := aColor.
	self flushCachedForm.
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: ''!
cursorWrapped: aNumber

	| sz |
	editor cursor ~= aNumber ifTrue: [
		editor cursor: aNumber.
		sz := data size.
		sz = 0
			ifTrue: [editor cursor: 1]
			ifFalse: [
				((editor cursor >= (sz + 1)) or: [editor cursor < 0]) ifTrue: [
					 editor cursor: editor cursor - ((editor cursor // sz) * sz)].
				editor cursor < 1 ifTrue: [editor cursor: sz + editor cursor]].
		"assert: 1 <= cursor < data size + 1"
		hasChanged := true].
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: ''!
cursor: aNumber


	editor cursor: aNumber.
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'jm 9/10/2004 20:21'!
customScale
	"Called when the user wants to input a scale value."

	| answer |
	answer := FillInTheBlank request: 'Please type desired scale:' initialAnswer: '2x'.
	answer size = 0 ifTrue: [^ self].
	answer := answer copyWithout: $x.
	self zoom: answer asNumber.
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'jm 7/12/2003 17:23'!
data

	^ data
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'jm 7/12/2003 17:23'!
dataColor

	^ dataColor
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'jm 7/12/2003 17:23'!
dataColor: aColor

	dataColor := aColor.
	self flushCachedForm.
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'jm 7/12/2003 17:23'!
data: aCollection

	data := aCollection.
	maxVal := minVal := 0.
	data do: [:x |
		x < minVal ifTrue: [minVal := x].
		x > maxVal ifTrue: [maxVal := x]].

	self flushCachedForm.
! !

!ScratchGraphMorph methodsFor: 'private' stamp: ''!
drawCursorOn: aCanvas

	| ptr x r c |
	ptr := (editor cursor asInteger max: 1) min: data size.
	c := cursorColor.
	((ptr > 1) and: [ptr < data size]) ifTrue: [
		(data at: ptr) sign ~= (data at: ptr + 1) sign
			ifTrue: [c := cursorColorAtZeroCrossings]].
	r := self innerBounds.
	x := r left + ((ptr - startIndex)*scale).
	((x >= r left) and: [x <= r right]) ifTrue: [
		aCanvas fillRectangle: (x@r top corner: x + 1@r bottom) color: c].
! !

!ScratchGraphMorph methodsFor: 'private' stamp: ''!
drawDataOn: aCanvas

	| x start end left right yScale baseLine top bottom |
	super drawOn: aCanvas.

	viewer ifTrue: [self drawViewOn: aCanvas.].
	self drawSelectionOn: aCanvas.

	(posVals isNil) ifTrue: [^ self].

	maxVal = minVal ifTrue: [yScale := 1.] 
					ifFalse: [yScale := (bounds height - (2 * borderWidth)) asFloat / (maxVal - minVal)].
	baseLine := bounds bottom - borderWidth + (minVal * yScale) truncated.

	left := 0. right := 10.
	x := bounds left + borderWidth.

	start := (startIndex*scale) truncated min: data size max: 1.
	end := start + bounds width min: data size.

	start to: end do: [:i |
		i > posVals size ifTrue: [^self].
					
		left := x truncated. right := x + 1.
		right > (bounds right - borderWidth) ifTrue: [^ self].

		top := baseLine min: (baseLine - (yScale*(posVals at: i))) truncated.
		bottom := (baseLine max: baseLine - (yScale*(negVals at: i))) truncated.
											
		aCanvas fillRectangle: (left@top corner: right@bottom) color: dataColor.
		x := x + 1].! !

!ScratchGraphMorph methodsFor: 'drawing' stamp: ''!
drawOn: aCanvas

	| c |

	cachedForm = nil ifTrue:  [
		c := FormCanvas extent: bounds extent.
		c translateBy: bounds origin negated
			during:[:tempCanvas| self drawDataOn: tempCanvas].
		cachedForm := c form].
	aCanvas paintImage: cachedForm at: bounds origin.
	self drawCursorOn: aCanvas.
	self drawPlayCursorOn: aCanvas.
	! !

!ScratchGraphMorph methodsFor: 'private' stamp: ''!
drawPlayCursorOn: aCanvas

	| ptr x r c |

	editor playCursor ifNil: [^self].
	(editor endPlaying ~= data size) & (editor playCursor >= (editor endPlaying)) ifTrue: [^self].

	ptr := (editor playCursor asInteger max: 1) min: data size.
	c := cursorColor.
	r := self innerBounds.
	x := r left + ((ptr - startIndex)*scale).
	((x >= r left) and: [x <= r right]) ifTrue: [
		aCanvas fillRectangle: (x@r top corner: x + 1@r bottom) color: c].
! !

!ScratchGraphMorph methodsFor: 'private' stamp: 'kfr 4/17/2014 23:36'!
drawSelectionOn: aCanvas

	| x y lightColor darkColor v1 v2 offsetX s1 s2 bottom |
editor ifNil: [^ self].
	editor selectionNil ifTrue: [^ self].

	lightColor := Color lightBlue. "(Color r: 0.2 g: 1.0 b: 0.907)."
	darkColor := lightColor darker darker darker.

	v1 := (editor graph startIndex asInteger max: 1) min: data size.
	v2 := v1 + (bounds width/(editor graph scale)) min: data size.
	

	offsetX := bounds left + borderWidth.
	x := (offsetX + ((selection first - startIndex)*scale)).
	y := bounds top + borderWidth.

	viewer 
		ifFalse: [
			selection first > v2 ifTrue: [^ self].  "selection is not visible"
			selection last < v1 ifTrue: [^ self].  "selection is not visible"

			aCanvas
				fillRectangle: (x@y extent: ((selection last - selection first)*scale)@(self height - (2 * borderWidth)))
				color: lightColor. "lightYellow"]
		ifTrue: [
			s1 := selection first.
			s2 := selection second.

			bottom := self height - (2 * borderWidth).


			((s1 max: s2) <= v1) | ((s1 min: s2) >= v2)
				ifTrue: [^aCanvas 				
							fillRectangle: ((offsetX + (s1*scale))@y extent: ((s2-s1)*scale)@bottom)
							color: darkColor.].
			
			(s1 <= v1) & (s2 >= v2)
				ifTrue: [^aCanvas fillRectangle: ((offsetX + (s1*scale))@y extent: ((v1-s1)*scale)@bottom)
								color: darkColor;
								fillRectangle: ((offsetX + (v1*scale))@y extent: ((v2-v1)*scale)@bottom)
								color: lightColor;
								fillRectangle: (( offsetX + (v2*scale))@y extent: ((s2-v2)*scale)@bottom)
								color: darkColor.].
			
			(s1 >= v1) & (s2 <= v2)
				ifTrue: [^aCanvas
							fillRectangle: ((offsetX + (s1*scale))@y extent: ((s2-s1)*scale)@bottom)
							color: lightColor.].
	
			(s1 < v1) & (s2 > v1) & (s2 < v2)
				ifTrue: [^aCanvas
							fillRectangle: ((offsetX + (s1*scale))@y extent: ((v1-s1)*scale)@bottom)
							color: darkColor;
							fillRectangle: ((offsetX + (v1*scale))@y extent: ((s2- v1)*scale)@bottom)
							color: lightColor.].
		
			(s1 >= v1) & (s2 >= v2) 
				ifTrue: [^aCanvas
							fillRectangle: ((offsetX + (s1*scale))@y extent: ((v2-s1)*scale)@bottom)
							color: lightColor;
							fillRectangle: ((offsetX + (v2*scale))@y extent: ((s2 - v2)*scale)@bottom)
							color: darkColor.].

			"Transcript show: 'no category :(';cr."

			].



! !

!ScratchGraphMorph methodsFor: 'private' stamp: ''!
drawViewOn: aCanvas

	"highlights the part of the graph morph we're viewing."
	| y ex start x |

	viewer ifFalse: [^self].
	
	
	start := ((editor graph startIndex*bounds width)/data size) truncated min: data size.
	ex :=  ((editor slider sliderThickness/editor slider extent x)*(bounds width)) min: data size.

	x := (bounds left + borderWidth + start).
	y := bounds top + borderWidth.

	aCanvas
		fillRectangle: (x@y extent: ex@(self height - (2 * borderWidth)))
		color: Color white. "lightYellow".
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: ''!
editor

	^editor! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: ''!
editor: aSoundEditor

	editor := aSoundEditor.! !

!ScratchGraphMorph methodsFor: 'private' stamp: 'LY 7/25/2003 17:31'!
flushCachedForm

	cachedForm := nil.
	hasChanged := true.
 ! !

!ScratchGraphMorph methodsFor: 'events' stamp: 'jm 7/12/2003 17:23'!
handlesMouseDown: evt

	^ true
! !

!ScratchGraphMorph methodsFor: 'drawing' stamp: 'LY 7/25/2003 17:31'!
hasChanged: aBoolean
	
	hasChanged := aBoolean.! !

!ScratchGraphMorph methodsFor: 'initialization' stamp: ''!
initialize

	super initialize.
	self color: (Color r: 0.8 g: 0.8 b: 0.6).
	self extent: 365@80.
	self borderWidth: 2.
	dataColor := Color darkGray.
	cursorColor := Color red.
	playCursorColor := Color blue.
	cursorColorAtZeroCrossings := Color red.
	startIndex := 1.
	viewer := false.
	selection := {nil. nil}.
	scale := 1.0.
	hasChanged := false.
	posVals := negVals := nil.

	self data:
		((0 to: 360 - 1) collect:
			[:x | (100.0 * (x degreesToRadians sin)) asInteger]).

	
	
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'jm 7/28/2003 10:11'!
interpolatedValueAtCursor

	| sz prev frac next |
	data isEmpty ifTrue: [^ 0].
	sz := data size.
	owner cursor < 0 ifTrue: [^ data at: 1].  "just to be safe, though cursor shouldn't be negative"
	prev := owner cursor truncated.
	frac := owner cursor - prev.
	prev < 1 ifTrue: [prev := sz].
	prev > sz ifTrue: [prev := 1].
	"assert: 1 <= prev <= sz"
	frac = 0 ifTrue: [^ data at: prev].  "no interpolation needed"

	"interpolate"
	next := prev = sz ifTrue: [1] ifFalse: [prev + 1].
	^ ((1.0 - frac) * (data at: prev)) + (frac * (data at: next))
! !

!ScratchGraphMorph methodsFor: 'private' stamp: 'LY 8/1/2003 13:18'!
keepIndexInView: index

	| w newStart |

	w := bounds width - (2 * borderWidth).
	index < startIndex ifTrue: [
		newStart := index - w//scale + 1.
		^ self startIndex: (newStart max: 1)].
	index > (startIndex + w//scale) ifTrue: [
		^ self startIndex: (index min: data size)].
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'jm 7/12/2003 17:23'!
lastValue

	data size = 0 ifTrue: [^ 0].
	^ data last
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'jm 7/12/2003 17:23'!
lastValue: aNumber

	self appendValue: aNumber.
! !

!ScratchGraphMorph methodsFor: 'change reporting' stamp: 'jm 7/12/2003 17:23'!
layoutChanged

	super layoutChanged.
	cachedForm := nil.
! !

!ScratchGraphMorph methodsFor: 'commands' stamp: 'jm 7/12/2003 17:23'!
loadSineWave

	self loadSoundData: FMSound sineTable.
! !

!ScratchGraphMorph methodsFor: 'commands' stamp: 'jm 2/3/2009 14:17'!
loadSoundData: aCollection

	| factor absV newData |
	factor := 0.
	aCollection do: [:v | (absV := v abs) > factor ifTrue: [scale := absV]].
	scale := 100.0 / factor.
	newData := OrderedCollection new: aCollection size.
	1 to: aCollection size do: [:i | newData addLast: (factor * (aCollection at: i))].

	self data: newData.
	self startIndex: 1.
	self cursor: 1.
! !

!ScratchGraphMorph methodsFor: 'commands' stamp: 'jm 7/12/2003 17:23'!
loadSound: aSound

	self loadSoundData: aSound samples.
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'kfr 4/18/2014 00:15'!
minScale
	
		
	"aNumber should be equal to: viewer extent x/ graph data size"
	minScale ifNil:[minScale := (self extent x / data size) asFloat].

	^minScale! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'kfr 4/18/2014 00:12'!
minScale: aNumber

	"aNumber should be equal to: viewer extent x/ graph data size"
	minScale := aNumber asFloat.
! !

!ScratchGraphMorph methodsFor: 'events' stamp: ''!
mouseDown: evt

	"Handles mouse down and drag events.  Updates the cursor's position and sets the selection to an array containing two copies of the current cursor value."
	| x s |

	 x := evt cursorPoint x - (bounds left + borderWidth).

	s := editor startSelection.
	editor startSelection: editor cursor.  
	editor cursor: startIndex + (x/scale).
		
	evt shiftPressed
		ifTrue: [  editor selectionNil 
					ifFalse: [
						editor startSelection: s.
				  		self adjustSelection.].]
		ifFalse: [
				 ((editor selectionNil not) and: [(selection at: 2) - (selection at: 1) > 3])
						ifTrue: [ editor selection: nil.
			  				      self flushCachedForm.
				 				 self changed.].
			  	 editor startSelection: editor cursor.
				 editor selection: {editor cursor. editor cursor}.].


	 
	
	! !

!ScratchGraphMorph methodsFor: 'events' stamp: ''!
mouseMove: evt

	"Updates the cursor position as the mouse moves.  Adjusts the selection only if the mouse is currently being pressed"
	| x w |

	x := evt cursorPoint x - (bounds left + borderWidth).
	w := self width - (2 * borderWidth).

	(viewer not and: [x < 0]) ifTrue: [
		editor cursor: startIndex + (x /scale).
		self adjustSelection.
		editor slider setValue: (startIndex/data size).
		^ self startIndex: self editor cursor].
	(viewer not and: [x > w]) ifTrue: [
		editor cursor: startIndex + (x /scale).
		self adjustSelection.
		editor slider setValue: (startIndex/data size).
		^ self startIndex: editor cursor - (w/scale) truncated.
		"^ editor cursor = data size
			ifTrue: [ self startIndex: editor cursor - (w/(scale*2)) truncated.]
			ifFalse: [ self startIndex: editor cursor - (w/scale) truncated.]."].


	evt anyButtonPressed 
		ifTrue: [editor cursor: (startIndex + (x/scale) truncated).
				self adjustSelection.]
! !

!ScratchGraphMorph methodsFor: 'events' stamp: ''!
mouseUp: evt

	((editor selectionNil not) and: [(selection at: 2) - (selection at: 1) <=3])
		ifTrue: [editor selection: nil.
				editor startSelection: nil.].
! !

!ScratchGraphMorph methodsFor: 'menu' stamp: 'tfel 8/9/2016 16:38'!
openWaveEditor

	| factor scaledData |
	self data: data.  "make sure maxVal and minVal are current"
	factor := 32767 // ((minVal abs max: maxVal abs) max: 1).
	scaledData := SoundBuffer newMonoSampleCount: data size.
	1 to: data size do: [:i | scaledData at: i put: (factor * (data at: i)) truncated].
	(WaveEditor new
		data: scaledData;
		samplingRate: 11025) openInWorld.
! !

!ScratchGraphMorph methodsFor: 'commands' stamp: 'jm 2/3/2009 14:19'!
playOnce

	| factor absV scaledData |
	data isEmpty ifTrue: [^ self].  "nothing to play"
	factor := 1.
	data do: [:v | (absV := v abs) > factor ifTrue: [factor := absV]].
	factor := 32767.0 / factor.
	scaledData := SoundBuffer newMonoSampleCount: data size.
	1 to: data size do: [:i |
		scaledData at: i put: (factor * (data at: i)) truncated].
	(SampledSound samples: scaledData samplingRate: 11025) play.
! !

!ScratchGraphMorph methodsFor: 'menu' stamp: 'kfr 4/17/2014 23:59'!
readDataFromFile

	| result fName |
	result := StandardFileMenu oldFile. "Extensions: #(aif aiff au wav)."
	result ifNil: [^ self].
	fName :=  result directory pathName, FileDirectory slash, result name.
	self data: (SampledSound fromFileNamed: fName) samples.

! !

!ScratchGraphMorph methodsFor: 'commands' stamp: 'jm 7/12/2003 17:23'!
reverse

	data := data reversed.
	self flushCachedForm.
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'LY 7/31/2003 12:09'!
ruler

	^ruler.! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'kfr 4/19/2014 14:17'!
ruler: aRuler

	ruler:=aRuler! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'LY 7/30/2003 10:54'!
scale
	
	^scale.! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'kfr 4/18/2014 00:17'!
scale: aNumber

	"setting the absolute scale of how the graph is display.  It is relative to MinScale, the minimum scale possible, which is the scale value of the viewer."

	aNumber < 1 ifTrue: [^self].
	scale := (aNumber* self minScale) asFloat min: 1.
	
	self calculateDataArray.
	self flushCachedForm; changed.
	editor viewer flushCachedForm; changed.

	editor updateSlider.
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'jm 7/12/2003 17:23'!
selection

	^ selection
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'LY 7/26/2003 16:48'!
selection: anArrayOrNil
	"Set the selection to the given (startIndex, stopIndex) pair to to nil."

	anArrayOrNil
		ifNil: [	selection at: 1 put: nil. selection at: 2 put: nil]
		ifNotNil: [ selection := anArrayOrNil.].

! !

!ScratchGraphMorph methodsFor: 'viewing' stamp: ''!
setScale

	| menu choice |

	menu := CustomMenu new.
	menu add: '1x' action: '1';
		   add: '2x' action: '2';
		   add: '3x' action: '3';
		   add: '4x' action: '4';
		   add: '5x' action: '5';
		   add: 'other' action: #customScale.
	choice := menu startUp.
	choice ifNil: [^self].

	choice = #customScale 
		ifFalse: [self scale: choice asNumber.]	
		ifTrue: [ self customScale].! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'jm 7/12/2003 17:23'!
startIndex

	^ startIndex
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: 'jm 7/12/2003 17:23'!
startIndex: aNumber

	startIndex ~= aNumber ifTrue:  [
		startIndex := aNumber asInteger.
		self flushCachedForm].
! !

!ScratchGraphMorph methodsFor: 'stepping' stamp: ''!
step
	"Make a deferred damage rectangle if I've changed. This allows applications to call methods that invalidate my display at high-bandwidth without paying the cost of doing the damage reporting on ever call; they can merely set hasChanged to true."

	super step.
	
	hasChanged == nil ifTrue: [hasChanged := false].
	hasChanged ifTrue: [
		self changed.
		hasChanged := false].
! !

!ScratchGraphMorph methodsFor: 'stepping' stamp: 'LY 7/26/2003 15:51'!
stepTime

	^150! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: ''!
valueAtCursor

	data isEmpty ifTrue: [^ 0].
	^ data at: ((editor cursor truncated max: 1) min: data size).
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: ''!
valueAtCursor: aPointOrNumber

	data isEmpty ifTrue: [^ 0].
	data
		at: ((editor cursor truncated max: 1) min: data size)
		put: (self asNumber: aPointOrNumber).
	self flushCachedForm.
! !

!ScratchGraphMorph methodsFor: 'accessing' stamp: ''!
viewer: aBoolean

	viewer := aBoolean. "whether or not this graphMorph is the viewer graph Morph"
	
	viewer ifFalse: [self color: Color white.].! !

!ScratchGraphMorph methodsFor: 'viewing' stamp: 'kfr 4/18/2014 00:17'!
viewSelection

	| diff |
	selection ifNil: [^self].
	scale := (self minScale*(data size/(selection second - selection first)) asFloat min: 1).
	self calculateDataArray.

	diff := (selection second - selection first) - bounds width.
	diff < 0
		ifTrue: [ self startIndex: selection first asInteger + (diff//2)]
		ifFalse: [self startIndex: selection first asInteger.].
	
	
	editor updateSlider.


	! !

!ScratchGraphMorph methodsFor: 'viewing' stamp: 'LY 7/30/2003 15:19'!
zoomIn

	self zoom: 2.! !

!ScratchGraphMorph methodsFor: 'viewing' stamp: 'LY 7/30/2003 15:21'!
zoomOut

	self zoom: 0.5.! !

!ScratchGraphMorph methodsFor: 'viewing' stamp: 'kfr 4/18/2014 00:16'!
zoom: aNumber

	scale := ((scale*aNumber asFloat) max: self minScale asFloat) min: 1.
	
	self calculateDataArray.
	self flushCachedForm; changed.
	editor viewer flushCachedForm; changed.

	editor updateSlider.! !

RectangleMorph subclass: #ScratchRulerMorph
	instanceVariableNames: 'max min marks graph sigDigs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScratchSoundEditor'!

!ScratchRulerMorph commentStamp: 'tpr 12/6/2013 18:23' prior: 0!
A ScratchRulerMorph is an apparently unused class!

!ScratchRulerMorph class methodsFor: 'instance creation' stamp: 'LY 7/23/2003 10:46'!
graphMorph: aScratchGraphMorph

	^ self new graph: aScratchGraphMorph; extent: (aScratchGraphMorph extent x)@20.

	! !

!ScratchRulerMorph methodsFor: 'drawing' stamp: 'kfr 4/17/2014 23:43'!
drawOn: aCanvas

	| space bigSpace currVal s ss |
	
	"the amt of space btw each bar.  big Space = the amt of space btw each big bar"
	space := (self extent x/marks) truncated max: 1.
	bigSpace := 5* space asFloat.

	graph ifNotNil: [ss := graph editor origSamplingRate ]
		   ifNil: [ss := 1].

ss ifNil:[ss:=1].
	currVal := (min/ss) asFloat roundTo: 0.01.  "the value of where we are in teh rule"

	aCanvas fillRectangle: (Rectangle origin: (self left)@(self top) corner: (self right)@(self bottom)) color: Color lightGray.
	self removeAllMorphs.

	(self left) to: (self right) by: space do: 
		[:pos |
			(pos - (self left)) \\ bigSpace = 0
				ifTrue: [aCanvas line: (pos truncated)@((self top) truncated) to: (pos truncated)@((self top + 5) truncated) color: Color black.
				s := StringMorph contents: (currVal asString).
				s center: (pos truncated)@(self top + 12).
				self addMorph: s.]
				ifFalse: [aCanvas line: (pos truncated)@(self top truncated) to: (pos truncated)@((self top + 1) truncated) color: Color black.]. 
			currVal := currVal + ((max-min)/(marks*ss)) roundTo: 0.01. ].




! !

!ScratchRulerMorph methodsFor: 'accessing' stamp: 'LY 7/23/2003 10:31'!
graph: aScratchGraphMorph

	graph := aScratchGraphMorph.! !

!ScratchRulerMorph methodsFor: 'initialize' stamp: 'LY 7/23/2003 10:30'!
initialize
	
	super initialize.
	min := 0.0.
	max := 100.
	marks := 20.0.
	sigDigs := 1. 
	graph := nil. "used specifically to coord with ScratchGraphMorph"

	self borderWidth: 1.
	self extent: 200@20.
	self color: Color white.! !

!ScratchRulerMorph methodsFor: 'accessing' stamp: 'LY 7/23/2003 10:21'!
marks: aNumber
	marks := aNumber.
	self changed.! !

!ScratchRulerMorph methodsFor: 'accessing' stamp: 'LY 7/23/2003 10:19'!
min: aMin max: aMax

	min := aMin.
	max := aMax.
	self changed.! !

!ScratchRulerMorph methodsFor: 'accessing' stamp: 'LY 7/23/2003 10:21'!
sigDigs: aNumber

	sigDigs := aNumber truncated.
	self changed.! !

!ScratchRulerMorph methodsFor: 'stepping' stamp: 'LY 7/31/2003 11:48'!
step
	
	| graphStart graphEnd |
	graph ifNil: [^self].

	self extent x = graph extent x
		ifFalse: [self extent: (graph extent x)@(self extent y).].


	graphStart := graph startIndex min: graph data size.
	graphEnd := graphStart + (graph bounds width/graph scale) min: graph data size.
	
	(min = graphStart and:[ max = graphEnd])
		ifFalse: [ min := graphStart.
				  max := graphEnd truncated.
				  self changed.].! !

AlignmentMorph subclass: #ScratchSoundEditor
	instanceVariableNames: 'client soundName graph viewer selection startSelection playCursor cursor origSamplingRate samplingRate rateSlider slider volume snd completeSnd soundMorphSound copy time scrollDelta deltaVal fwdButton backButton undoSound undoSel undoCursor undoScale endPlaying'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ScratchSoundEditor'!

!ScratchSoundEditor commentStamp: 'tpr 12/6/2013 18:23' prior: 0!
A SCratchSoundEditor is an apparently unused class.

A sound editor, based on the wave editor.  It can be brought up by the SoundMorph.  When the soundEditor is brought up, it is created with a copy of the sound in the sound morph. It contains the sound of the soundmorph and is capable of editing that sound.  That sound is set in the method "sound:"

"viewing"
graph 					GraphMorph			
viewer					GraphMorph
selection 				    Array				an array of 2 #s that are the start and end of the selection.  This and the next 2 												variables are in terms of indices of the graph data
startSelection 			   Number				a number indicating the constant pt in a selection (mouse down loc)
cursor					   Number				a number indicating the location of the cursor

"Scrolling in view"
scrollDelta			        Number				a number telling us how much to scroll by, used in stepping to allow scrolling by 												buttons
slider					   slider morph			a slider for the graph
time					   a number			where we are in the graph in terms os msecs
deltaVal 					number				how much the cursor scrolls by, scrolldelta is set to +- this value
fwdButton 					button				scolling
backButton     				button				scrolling

"Playing variables" 
origSamplingRate 		  big number			indicates the sampling rate of the song @ loading
												so u can reset to that sampling rate.
samplingRate 	         another big number		indicates current sampling rate.	
(2 above aren't currently used)

volume					   slider morph			indicates volume, (0 to 1.0)		
preview						boolean 			whether we're in play all mode or play 	from 												cursor mode, where the cursor moves along with 												the music
"Sound manipulation"
snd 						SampledSound		Current sound used for playing from cursor + selection
completeSnd 				SampledSound		The entire song after editing (doesn't change with playing)
soundMorphSound   			SampledSound		a ref to the sound of the SoundMorph that created this, 
												so that u can change the sound within that sound editor

"Editing tools"
copy						SoundBuffer		the portion of the graph data that has been copied
undoSound 					SampledSound		the sound before any cuts/pastes crops
undoSel 					Array				a copy of the selection bf any cut/paste/crop


rateSlider 					currently not implemented
keyboard!

!ScratchSoundEditor class methodsFor: 'icons' stamp: 'kfr 4/20/2014 12:33'!
descriptionForPartsBin
	^ self partName:	'SoundEditor' translatedNoop
		categories:		{'Multimedia' translatedNoop}
		documentation:	'A workbench for seing and editing wave forms' translatedNoop! !

!ScratchSoundEditor class methodsFor: 'icons' stamp: 'jm 5/6/2004 19:39'!
downArrow

	^ Form
		extent: 10@12
		depth: 4
		fromArray: #(3722304989 3707764736 3722304989 3707764736 3704479196 3170893824 3692739489 3170893824 3550548241 1023410176 3720417563 3707764736 3711570339 3707764736 3722121645 3707764736 3722252605 3707764736 3722296285 3707764736 3722261469 3707764736 3722304989 3707764736)
		offset: 0@0
! !

!ScratchSoundEditor class methodsFor: 'instance creation' stamp: 'LY 7/1/2003 14:14'!
openOn: dataCollection
	"Open a new WaveEditor on the given sequencable collection of data."

	^ (self new data: dataCollection) openInWorld
! !

!ScratchSoundEditor class methodsFor: 'icons' stamp: 'jm 5/6/2004 19:43'!
upArrow
	"Uparrow form used in slider." 

	^ Form
		extent: 6@3
		fromArray: #(2r11e28 2r1111e27 2r111111e26)
		offset: 0@0
! !

!ScratchSoundEditor methodsFor: 'initialization' stamp: 'kfr 4/20/2014 12:18'!
addControls

	| b r spacer |
	b := SimpleButtonMorph new target: self; borderColor: Color black; useSquareCorners.
	b borderColor: #raised; borderWidth: 3.
	r := AlignmentMorph newRow.
	r color: Color transparent; borderWidth: 0; layoutInset: 0.
	r hResizing: #spaceFill; vResizing: #shrinkWrap; extent: 5@5.
	r wrapCentering: #center.

	spacer := Morph new color: r color; extent: 40@5.  "spacer"
	r addMorphBack: (spacer fullCopy width: 5).
	r addMorphBack: self dismissButton.
	r addMorphBack: (spacer fullCopy width: 5).
	r addMorphBack: (b fullCopy label: 'undo';		actionSelector: #undo).
	r addMorphBack: (spacer fullCopy width: 5).
	r addMorphBack: (b fullCopy label: 'copy';		actionSelector: #copy).
	r addMorphBack: (spacer fullCopy width: 5).
	r addMorphBack: (b fullCopy label: 'paste';		actionSelector: #paste).
	r addMorphBack: (spacer fullCopy width: 5).
	r addMorphBack: (b fullCopy label: 'cut';		actionSelector: #cut).
	r addMorphBack: (spacer fullCopy width: 5).
	r addMorphBack: (b fullCopy label: 'crop';		actionSelector: #crop).	
	r addMorphBack: (spacer fullCopy width: 25).
	
	r addMorphBack: (b fullCopy label: 'save';			actionSelector: #saveToSoundLibrary).
	r addMorphBack: (spacer fullCopy width: 5).
	

	self addMorphBack: r.

! !

!ScratchSoundEditor methodsFor: 'initialization' stamp: 'tpr 7/26/2013 18:00'!
addGraph

	| r |
	r := AlignmentMorph newRow.
	r color: Color transparent; borderWidth: 0; layoutInset: 0.
	r hResizing: #shrinkWrap; vResizing: #shrinkWrap; extent: 5@5.
	r wrapCentering: #center.

	r addMorphBack: (AlignmentMorph newSpacer: r color).
	r addMorphBack: graph.
	r addMorphBack: (AlignmentMorph newSpacer: r color).
	self addMorphBack: r.
! !

!ScratchSoundEditor methodsFor: 'initialization' stamp: 'kfr 4/20/2014 12:17'!
addPlayButtons

	| b r m space n scaleSelector |
	b := SimpleButtonMorph new target: self; borderColor: Color black; useSquareCorners.
	b borderColor: #raised; borderWidth: 3.
	r := AlignmentMorph newColumn.
	r color: Color transparent; borderWidth: 0; layoutInset: 0.
	r hResizing: #spaceFill; vResizing: #shrinkWrap; extent: 5@5.
	r wrapCentering: #center.

	m := AlignmentMorph newRow.
	m color: Color transparent; borderWidth: 0; layoutInset: 0.
	m hResizing: #shrinkWrap; vResizing: #shrinkWrap; extent: 5@5.
	m wrapCentering: #center.

	space := Morph new color: r color; extent: 3@5.

	n := StringMorph new contents: 'Zoom to: '.
	m addMorphBack: n.

	scaleSelector :=  (IconicButton new
		labelGraphic: self class downArrow;
		extent: 15@15;
		color: Color transparent;
		borderWidth: 0;
		target: graph;
		actWhen: #buttonDown;
		actionSelector: #setScale).
	m addMorphBack: scaleSelector.
	m addMorphBack: (Morph new color: r color; extent: 5@5).
	m addMorphBack: (b copy target: graph; label: '+';			actionSelector: #zoomIn).
	
	m addMorphBack: (b copy target: graph; label: '-';			actionSelector: #zoomOut).
	m addMorphBack: (b copy target: graph;
							label: 'zoom selection';	actionSelector: #viewSelection).

	m addMorphBack: (Morph new color: r color; extent: 5@5).
	"n := UpdatingStringMorph new
	target: graph;
	getSelector: #scale; 
	growable: false; width: 25; step.
	m addMorphBack: n."


	m addMorphBack: (Morph new color: r color; extent: 50@5).
	
	
	m addMorphBack: (b fullCopy label: 'Play All';			actionSelector: #playAll).
	m addMorphBack: space copy.
	m addMorphBack: (b fullCopy label: 'Play';				actionSelector: #playAfterCursor).
	m addMorphBack: space copy.
	m addMorphBack: (b fullCopy label: 'Stop';				actionSelector: #stopProcess).

	m addMorphBack: (Morph new color: r color; extent: 50@5).
	m addMorphBack: (StringMorph new contents: 'Volume').
	m addMorphBack: space copy.
	
	volume := SimpleSliderMorph new
	color: Color veryVeryLightGray;
	extent: 60@20;
	target: self;
	actionSelector: #setVolume:.

	m addMorphBack: volume.

	m addMorphBack: (Morph new color: r color; extent: 50@5).

	r addMorphBack: (Morph new color: r color; extent: 5@5).
	r addMorphBack: m.
	

	self addMorphBack: r.

! !

!ScratchSoundEditor methodsFor: 'initialization' stamp: 'kfr 4/20/2014 11:24'!
addRuler

	| ruler r |


	r := AlignmentMorph newRow.
	r color: Color transparent; borderWidth: 0; layoutInset: 0.
	r hResizing: #shrinkWrap; vResizing: #shrinkWrap; extent: 5@5.
	r wrapCentering: #center.

	r addMorphBack: (AlignmentMorph newSpacer: r color).
	ruler := ScratchRulerMorph graphMorph: self.
	self addMorphBack: ruler.
	"r addMorphBack: (AlignmentMorph newSpacer: r color).
	self addMorphBack: r."
	graph ruler: ruler

	
	! !

!ScratchSoundEditor methodsFor: 'initialization' stamp: 'kfr 4/20/2014 18:19'!
addSlider

	| r spacer m |
	r := AlignmentMorph newRow.
	r color: Color transparent; borderWidth: 0; layoutInset: 0.
	r hResizing: #spaceFill; vResizing: #shrinkWrap; extent: 5@5.
	r wrapCentering: #center.	
	
	spacer := Morph new color: r color; extent: (50)@5.
	backButton := self makeSliderButton: 10@13.
	backButton addMorphCentered: (ImageMorph new image: (self class upArrow rotateBy: #left centerAt: 0@0)).

	fwdButton := self makeSliderButton: 10@13.
	fwdButton addMorphCentered: (ImageMorph new image: (self class upArrow rotateBy: #right centerAt: 0@0)).
	
	slider := SimpleSliderMorph new
		color: Color veryVeryLightGray;
		extent: (graph width)@5;
		target: self;
		setMaxVal: (graph bounds width/(graph scale*graph data size));
		actionSelector: #scrollTime:.
	
	r addVariableTransparentSpacer.
	r addMorphBack:backButton.
	r addMorphBack: slider.
	r addMorphBack: fwdButton.
	r addVariableTransparentSpacer.
	m := Morph new color: r color; extent: 15@5.  "spacer"
	"r addMorphBack: m."
	time := UpdatingStringMorph new
		target: self;
		getSelector: #startTime; 
		width: 40; step.
	time useStringFormat.
	"r addMorphBack: time."

	self addMorphBack: r.
! !

!ScratchSoundEditor methodsFor: 'initialization' stamp: 'kfr 4/20/2014 16:55'!
addValueSelectors

	| r m b |
	b := SimpleButtonMorph new target: self; borderColor: Color black; useSquareCorners.
	b borderColor: #raised; borderWidth: 3.
	r := AlignmentMorph newRow.
	r color: Color transparent; borderWidth: 0; layoutInset: 0.
	r hResizing: #spaceFill; vResizing: #shrinkWrap; extent: 5@5.
	r wrapCentering: #center.

	r addMorphBack: (Morph new color: r color; extent: 5@5).

	r addMorphBack: (b fullCopy label: '<slower';					actionSelector: #slower).
	r addMorphBack: (b fullCopy label: 'reset play rate';	actionSelector: #resetSamplingRate).
	r addMorphBack: (b fullCopy label: 'faster>';					actionSelector: #faster).

	r addMorphBack: (Morph new color: r color; extent: 25@5).
	m := StringMorph new contents: 'sampling rate'.
	r addMorphBack: m.
	m :=  Morph new color: r color; extent: 5@5. "spacer"
	r addMorphBack: m.

	rateSlider := SimpleSliderMorph new
	color: Color veryVeryLightGray;
	extent: 60@20;
	target: self;
	minVal: 0.2;
	maxVal: 2.5;
	actionSelector: #samplingRate:.
	r addMorphBack: rateSlider.
	r addMorphBack: (Morph new color: r color; extent: 30@5).

	m := StringMorph new contents: 'index: '.
	r addMorphBack: m.
	m := UpdatingStringMorph new
		target: graph; getSelector: #cursor; putSelector: #cursor:;
		growable:false; minimumWidth: 60; step.
	r addMorphBack: m.
	m fitContents.

	m :=  Morph new color: r color; extent: 20@5. "spacer"
	r addMorphBack: m.

	m := StringMorph new contents: 'sample value: '.
	r addMorphBack: m.
	m := UpdatingStringMorph new
		target: graph; getSelector: #valueAtCursor; putSelector: #valueAtCursor:;
		growable: false; minimumWidth: 60; step.
	r addMorphBack: m.
	m fitContents.

	m :=  Morph new color: r color; extent: 20@5. "spacer"
	r addMorphBack: m.

	

	self addMorphBack: r.

! !

!ScratchSoundEditor methodsFor: 'initialization' stamp: 'tpr 7/26/2013 18:00'!
addViewer

	| r |

	r := AlignmentMorph newRow.
	r color: Color transparent; borderWidth: 0; layoutInset: 0.
	r hResizing: #shrinkWrap; vResizing: #shrinkWrap; extent: 5@5.
	r wrapCentering: #center.

	r addMorphBack: (AlignmentMorph newSpacer: r color).
	r addMorphBack: viewer.
	r addMorphBack: (AlignmentMorph newSpacer: r color).

	self addMorphBack: r.
	
! !

!ScratchSoundEditor methodsFor: 'graph ops' stamp: 'LY 8/2/2003 17:13'!
back
	"Moves the cursor back in the music score."

	scrollDelta := -5.  
	self cursor: cursor + scrollDelta.! !

!ScratchSoundEditor methodsFor: 'graph ops' stamp: 'jm 8/23/2003 14:40'!
cancel

	self delete.
! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: 'jm 5/15/2004 21:54'!
client: anObject

	client := anObject.
! !

!ScratchSoundEditor methodsFor: 'graph ops' stamp: 'LY 7/24/2003 15:34'!
copy

	"copies the current selection if there is one."
	(selection at: 1) ifNotNil:
		[copy := graph data copyFrom: (graph selection at: 1) to: (graph selection at: 2).]! !

!ScratchSoundEditor methodsFor: 'graph ops' stamp: ''!
crop
	
	| sel |

	"Crops the sound in the sound editor to be only the selected region"

	sel := ((selection at: 1) notNil and: [(selection at: 2) - (selection at: 1) > 3]).
	sel ifFalse: [^self].

	undoSel := graph selection copy.
	undoSound := SampledSound samples: graph data samplingRate: samplingRate.
	undoCursor := cursor.
	undoScale := graph scale/ScratchGraphMorph MinScale.

	sel := graph data copyFrom: (selection at: 1) to: (selection at: 2).
	graph data: sel.
	viewer data: graph data.
	cursor := ((cursor - (selection at: 1)) truncated max: 1) min: graph data size truncated.

	"scales the botton viewer so that the data fits on the entire screen." 
	(selection second - selection first)/graph data size >= 0.05
		ifTrue: [ScratchGraphMorph MinScale: (viewer extent x/ graph data size min: 1). "the mult factor in all scales, the minimum scale allowed...to get this just do self scale: 1, since this multiplied by input"
				viewer scale: 1.
				graph scale: 1.
				viewer startIndex: 1.
				graph computeSlider.
				self fixSliderRange.].

	graph calculateDataArray.
	viewer calculateDataArray.

	snd setSamples: sel samplingRate: samplingRate.
	completeSnd setSamples: sel samplingRate: samplingRate.

	selection at: 1 put: 1.
	selection at: 2 put: graph data size.
! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: 'LY 7/25/2003 10:41'!
cursor

	^cursor.! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: ''!
cursor: aNumber

	graph data ifNil: [^self].

	cursor ~= aNumber ifTrue:  [
	cursor := ((aNumber truncated max: 1) min: graph data size) truncated.
	"graph keepIndexInView: cursor."
	].! !

!ScratchSoundEditor methodsFor: 'graph ops' stamp: ''!
cut 

	| data1 data2 |
	(selection at: 1) ifNil: [^ self.].

	undoSound := SampledSound samples: graph data samplingRate: samplingRate.
	undoSel := graph selection copy.
	undoCursor := cursor.
	undoScale := graph scale/ScratchGraphMorph MinScale.

	self copy.
	data1 := graph data copyFrom: 1 to: (selection at: 1).
	data2 := graph data copyFrom: (selection at: 2) to: graph data size.
	
	graph data: (data1, data2).
	viewer data: graph data.
		
	"scales the botton viewer so that the data fits on the entire screen." 
	((selection second - selection first)/graph data size asFloat) >= 0.05
		ifTrue: [ScratchGraphMorph MinScale: (viewer extent x/ graph data size min: 1).
				viewer scale: 1.
				graph scale: undoScale.
				viewer startIndex: 1.
				graph computeSlider.
				self fixSliderRange.].

	graph calculateDataArray.
	viewer calculateDataArray.

	snd setSamples: graph data samplingRate: samplingRate.
	completeSnd setSamples: graph data samplingRate: samplingRate.
	cursor := (selection at: 1).
	
	selection at: 1 put: nil. 
	selection at: 2 put: nil.! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: 'LY 7/1/2003 14:14'!
data: newData

	graph data: newData.
! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: ''!
endPlaying

	^endPlaying.! !

!ScratchSoundEditor methodsFor: 'menu' stamp: 'LY 7/10/2003 16:34'!
faster

	| rate |
	rate := rateSlider getScaledValue*1.1.
	rateSlider setScaledValue: rate.
	self samplingRate: rate.

	! !

!ScratchSoundEditor methodsFor: 'graph ops' stamp: ''!
fixSliderRange

	slider maxVal: (1 -(slider sliderThickness/slider extent x)).
	^slider! !

!ScratchSoundEditor methodsFor: 'graph ops' stamp: 'LY 7/26/2003 15:32'!
fwd
	
	"moves the cursor forward in the music."
	scrollDelta := 5.
	self cursor: cursor + 5.! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: 'LY 7/1/2003 14:14'!
graph

	^ graph
! !

!ScratchSoundEditor methodsFor: 'event handling' stamp: 'LY 7/7/2003 21:39'!
handlesMouseDown: evt

	| p |
	p := evt cursorPoint.
	^ (Rectangle origin: backButton topLeft corner: fwdButton bottomRight) containsPoint: p.! !

!ScratchSoundEditor methodsFor: 'event handling' stamp: 'LY 7/9/2003 12:38'!
handlesMouseOver: evt

	^true.! !

!ScratchSoundEditor methodsFor: 'initialization' stamp: 'kfr 4/20/2014 12:21'!
initialize

	super initialize.
	copy := nil.
	selection := {nil. nil}.
	scrollDelta := 0.
	deltaVal := 10.
	cursor := 200.
	playCursor := nil.
	samplingRate := SoundPlayer samplingRate.
	soundName := 'sound'.
	self extent: 5@5;
		listDirection: #topToBottom;
		wrapCentering: #centered;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		layoutInset: 3;
		color: Color lightGray;
		borderWidth: 2.
	graph := ScratchGraphMorph new extent: 380@150.
	graph editor: self.
	graph selection: selection.
	graph viewer: false.

	viewer := ScratchGraphMorph new extent: 380@30.
	viewer editor: self.
	viewer cursorColorAtZeroCrossings: Color red.
	viewer viewer: true.
	viewer selection: selection.

	self addControls.
	self addPlayButtons.
	self addValueSelectors.
	self addMorphBack: (Morph new color: self color; extent: 10@5).

	self addGraph.
	self addMorphBack: (Morph newBounds: (0@0 extent: 0@3) color: Color transparent).
	self addSlider.

	self addViewer.
	viewer left: backButton right.
! !

!ScratchSoundEditor methodsFor: 'menu' stamp: 'jm 7/12/2003 17:39'!
invokeMenu
	"Invoke a menu of additonal functions."

	| aMenu |
	aMenu := CustomMenu new.
	aMenu addList:	#(
		('save to file'		saveToFile)
		('read from file'	readFromFile)).
	aMenu invokeOn: self defaultSelection: nil.

! !

!ScratchSoundEditor methodsFor: 'event handling' stamp: 'LY 7/25/2003 11:27'!
keyStroke: evt

	| keyVal |
	keyVal := evt keyCharacter asciiValue.

	keyVal = 28 ifTrue: [ cursor := cursor + (( -10) / graph scale)].
	keyVal = 29 ifTrue: [ cursor := cursor + (10/graph scale)].! !

!ScratchSoundEditor methodsFor: 'initialization' stamp: 'jm 2/3/2009 14:27'!
makeSliderButton: buttonExtent

	| button |
	button := BorderedMorph
		newBounds: (self innerBounds bottomRight - buttonExtent extent: buttonExtent)
		color: Color lightGray.

	button setBorderWidth: 1 borderColor: #raised.
	^ button
! !

!ScratchSoundEditor methodsFor: 'event handling' stamp: 'LY 7/8/2003 11:55'!
mouseDown: evt

	| p |
	"do stuff"
	p := evt cursorPoint.
	

	(slider containsPoint: p) ifTrue: [ 
		slider descending ifTrue: [scrollDelta := deltaVal negated.]
						  ifFalse: [scrollDelta := deltaVal.].].
	(backButton containsPoint: p) ifTrue: [ backButton borderInset. scrollDelta := deltaVal negated ].
	(fwdButton containsPoint: p) ifTrue: [ fwdButton borderInset. scrollDelta := deltaVal].! !

!ScratchSoundEditor methodsFor: 'event handling' stamp: 'LY 7/9/2003 12:40'!
mouseEnter: evt

	evt hand newKeyboardFocus: self.! !

!ScratchSoundEditor methodsFor: 'event handling' stamp: 'LY 7/8/2003 13:16'!
mouseUp: evt

	"do stuff"
	scrollDelta := 0.
	fwdButton borderRaised.
	backButton borderRaised.
! !

!ScratchSoundEditor methodsFor: 'other' stamp: 'LY 7/1/2003 14:14'!
normalize: sampleArray
	"Return a copy of the given sample array scaled to use the maximum 16-bit sample range. Remove any D.C. offset."

	| max abs scale out |
	max := 0.
	sampleArray do: [:s |
		s > 0 ifTrue: [abs := s] ifFalse: [abs := 0 - s].
		abs > max ifTrue: [max := abs]].
	scale := ((1 << 15) - 1) asFloat / max.

	out := sampleArray species new: sampleArray size.
	1 to: sampleArray size do: [:i |
		out at: i put: (scale * (sampleArray at: i)) truncated].
	^ out
! !

!ScratchSoundEditor methodsFor: 'menu' stamp: 'LY 7/10/2003 14:02'!
origSamplingRate

	^origSamplingRate.
! !

!ScratchSoundEditor methodsFor: 'graph ops' stamp: ''!
paste
	| data1 data2 |
	"inserts the copied data into the dataset where the cursor is."
	
	copy ifNil: [^self ].
	
	"self selectionNil ifTrue: [undoSel := nil.] ifFalse: [undoSel := selection copy.]."
	undoSel := selection copy.
	undoSound := SampledSound samples: graph data samplingRate: samplingRate.
	undoCursor := cursor.
	undoScale := graph scale/ScratchGraphMorph MinScale.


	((self selectionNil not) and: [(selection at: 2) - (selection at: 1) > 3])
		ifTrue: [data1 := graph data copyFrom: 1 to: (graph selection at: 1).
			     data2 := graph data copyFrom: (graph selection at: 2) to: graph data size.]
		ifFalse: [data1 := graph data copyFrom: 1 to: graph cursor truncated.
				 data2 := graph data copyFrom: graph cursor truncated + 1 to: graph data size.].
	
	graph data: (data1, copy, data2).
	viewer data: graph data.

	"scales the botton viewer so that the data fits on the entire screen." 
	(copy size)/graph data size >= 0.05
		ifTrue: [ScratchGraphMorph MinScale: (viewer extent x/ graph data size min: 1).
				viewer scale: 1.
				graph scale: undoScale.
				self fixSliderRange.
				viewer startIndex: 1.
				graph computeSlider.].

	

	graph calculateDataArray.
	viewer calculateDataArray.

	snd setSamples: graph data samplingRate: samplingRate.
	completeSnd setSamples: graph data samplingRate: samplingRate.

	self selection: {data1 size. (data1 size) + (copy size).}.
	cursor := selection at: 2.
	graph startIndex: (slider getScaledValue)*(graph data size).

	viewer flushCachedForm.
		! !

!ScratchSoundEditor methodsFor: 'menu' stamp: 'LY 7/21/2003 13:45'!
pause

	(snd notNil and: [snd isPlaying])
		ifTrue: [snd pause.].
! !

!ScratchSoundEditor methodsFor: 'initialization' stamp: 'kfr 4/18/2014 00:25'!
playAfterCursor

	| sel currSel |
	graph data size < 2 ifTrue: [^ self].
	

	sel := ((selection at: 1) notNil and: [(selection at: 2) - (selection at: 1) > 3]).
	sel ifTrue: [currSel := graph data copyFrom: (selection at: 1) asInteger to: (selection at: 2) asInteger. playCursor := selection at: 1.
			    endPlaying := selection at: 2.]
	    ifFalse: [currSel := graph data copyFrom: (cursor max: 1) asInteger to: graph data size.
				playCursor := cursor.
				endPlaying := graph data size].
	
		self changed.


	snd isNil
		ifTrue: [
				 snd := (SampledSound samples: currSel samplingRate: samplingRate).
				 snd play.]
		ifFalse: [
			currSel = snd samples
				ifTrue: [snd samplesRemaining = 0 ifTrue: [snd reset].
						 snd resumePlaying.]
				ifFalse: [ snd setSamples: currSel samplingRate: samplingRate.
						 snd play]].

! !

!ScratchSoundEditor methodsFor: 'menu' stamp: 'kfr 4/18/2014 00:07'!
playAll

	"Resumes playing the selection if there is one, otherwise resumes playing the entire soundtrack."

	graph data size < 2 ifTrue: [^ self].
	playCursor := 1.
	endPlaying := graph data size.

	snd isNil
		ifTrue: [
				 snd := (SampledSound samples: graph data samplingRate: samplingRate).
				 snd play.]
		ifFalse: [
			snd reset.
			(graph data = snd samples)
				ifTrue: [snd samplesRemaining = 0 ifTrue: [snd reset].
						 snd resumePlaying.]
				ifFalse: [snd setSamples: graph data samplingRate: origSamplingRate.
						 snd play]]
! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: ''!
playCursor
	
	^playCursor.! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: ''!
playCursor: aNumber

	graph data ifNil: [^self].

	"allows the graph to set where the playing cursor is"

	cursor ~= aNumber ifTrue:  [
	cursor := ((aNumber truncated max: 1) min: graph data size) truncated.
	].! !

!ScratchSoundEditor methodsFor: 'menu' stamp: 'LY 7/18/2003 14:00'!
playFrom: start to: end

	| sz i1 i2 snd2 |
	sz := graph data size.
	i1 := ((start + 1) min: sz) max: 1.
	i2 := ((end + 1) min: sz) max: i1.
	(i1 + 2) >= i2 ifTrue: [^ self].
	snd2 := SampledSound
		samples: (graph data copyFrom: i1 to: i2)
		samplingRate: samplingRate.
	snd2 play.
! !

!ScratchSoundEditor methodsFor: 'menu' stamp: 'jm 12/4/2003 20:19'!
readFromFile
	"Read my samples from a file selected by the user."

	| result |
	result := StandardFileMenu oldFileExtensions: #(aif aiff au wav).
	result ifNil: [^ self].
	self readFromFile: result directory pathName, FileDirectory slash, result name.
! !

!ScratchSoundEditor methodsFor: 'menu' stamp: 'ee 11/15/2007 17:23'!
readFromFile: fName
	"Read my samples from the file with the given name."

	(FileDirectory default fileExists: fName)
		ifFalse: [^ self inform: 'File not found' withDetails: fName].
	snd := SampledSound fromFileNamed: fName.

	completeSnd := snd copy.
	graph cursor: 200.
	samplingRate := snd originalSamplingRate.
	origSamplingRate := snd originalSamplingRate.
	graph data: snd samples.
	graph scale: 2.0.

	self setVolume: 0.5.
	volume setScaledValue: 0.5.
	
	graph data size > 5000
		ifTrue: [deltaVal := 50]
		ifFalse: [deltaVal := graph data size // 10].

! !

!ScratchSoundEditor methodsFor: 'menu' stamp: 'LY 7/18/2003 14:20'!
resetSamplingRate

	snd ifNil: [^self].

	snd setSamples: snd samples samplingRate: origSamplingRate.
	rateSlider setScaledValue: 1.0.! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: 'LY 7/1/2003 14:14'!
samplingRate

	^ samplingRate

! !

!ScratchSoundEditor methodsFor: 'menu' stamp: 'LY 7/18/2003 14:17'!
samplingRate: sampleRate

"Sets the samplingRate to somewhere btw 0.1 and 2.5 the original samplingRate, given a number btw 0.2 and 2.5 (sampleRate)"

	snd ifNil: [^self.].

	origSamplingRate = 0 
		ifFalse: [snd setSamples: snd samples samplingRate: (sampleRate*origSamplingRate)]
		ifTrue: [ snd setSamples: snd samples samplingRate: (sampleRate*10000)].

	
! !

!ScratchSoundEditor methodsFor: 'graph ops' stamp: 'jm 6/4/2004 10:15'!
save

	snd := SampledSound samples: completeSnd samples samplingRate: samplingRate.
	client ifNotNil: [client saveSound: snd name: soundName].
	self delete.
! !

!ScratchSoundEditor methodsFor: 'menu' stamp: 'tfel 8/10/2016 10:09'!
saveToFile
	"Export my samples to a WAV file."

	| fileName samples f |
	fileName := UIManager default request: 'File name?' initialAnswer: ''.
	(fileName isNil or: [fileName size = 0]) ifTrue: [^ self].
	(fileName asLowercase endsWith: '.wav') ifFalse: [fileName := fileName, '.wav'].
	(graph data isKindOf: SoundBuffer)
		ifTrue: [samples := graph data]
		ifFalse: [samples := SoundBuffer fromArray: graph data].
	f := (FileStream newFileNamed: fileName) binary.
	(SampledSound samples: samples samplingRate: samplingRate) storeWAVSamplesOn: f.
	f close.
! !

!ScratchSoundEditor methodsFor: 'menu' stamp: 'kfr 4/20/2014 12:13'!
saveToSoundLibrary
	"The user hit the 'save' button."

	| sndName |
	
	self pause.

	sndName _ FillInTheBlank
				request: 'Sound name?' translated
				initialAnswer: 'unnamed' translated .
			sndName isEmpty ifTrue: [^ self].
			
	sndName := SampledSound unusedSoundNameLike: sndName.
	SampledSound
			addLibrarySoundNamed: sndName
			samples:  snd samples
			samplingRate:  samplingRate.
	

	! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: ''!
scrollTime: relativeValue

	graph startIndex: relativeValue*(graph data size).

	viewer flushCachedForm; changed.




! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: 'LY 7/25/2003 17:54'!
selection

	^selection.! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: 'LY 7/25/2003 17:53'!
selectionNil
	
	(selection at: 1) ifNil: [^ true].
	
	^false.! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: 'LY 7/25/2003 21:25'!
selection: anArrayorNil

	anArrayorNil ifNil: [selection := {nil. nil}.]
			 ifNotNil: [selection := anArrayorNil.].
	graph selection: selection.
	viewer selection: selection! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: 'LY 7/25/2003 23:19'!
selection: aNumber scd: anotherNumber

	selection := { aNumber min: anotherNumber. aNumber max: anotherNumber}.
	graph selection: selection.
	viewer selection: selection.
	graph flushCachedForm; changed.
	viewer flushCachedForm; changed.! !

!ScratchSoundEditor methodsFor: 'menu' stamp: 'LY 7/10/2003 17:16'!
setVolume: aFloat

	"aFloat is a value btw 0 and 1.0"
	
	snd ifNil: [^self].
	snd adjustVolumeTo: aFloat overMSecs: 50.! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: 'LY 7/30/2003 17:03'!
slider 
	^slider.! !

!ScratchSoundEditor methodsFor: 'menu' stamp: 'LY 7/10/2003 16:34'!
slower

	| rate |
	rate := rateSlider getScaledValue/1.1.
	rateSlider setScaledValue: rate.
	self samplingRate: rate.

	! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: 'LY 7/18/2003 12:42'!
sound

	^snd.! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: 'jm 6/4/2004 10:12'!
soundName: aString
	"Note the sound name to use when saving."

	soundName := aString.
! !

!ScratchSoundEditor methodsFor: 'graph ops' stamp: ''!
sound: aSampledSound

	"completeSnd is a version of the sound that is unaffected by the play mechanism.  This method is called when a SoundMorph brings up a SoundEditor. soundMorphSound is a copy of the initial sampledSound, used for saving and cancelling edits purposes.  It serves as a reference to the original sound so that we can actually change the sound in the soundMorph"

	soundMorphSound := aSampledSound.
	snd := aSampledSound copy.
	completeSnd := snd copy.
	graph cursor: 200.
	samplingRate := snd originalSamplingRate.
	origSamplingRate := snd originalSamplingRate.

	ScratchGraphMorph MinScale: (graph extent x/completeSnd samples size).

	graph data: completeSnd samples.  "SHOULD IT BE COPY?/"
	graph scale: 2.
	graph calculateDataArray.
	"self fixSliderRange.
	graph computeSlider."
	
	viewer data: completeSnd samples.
	viewer scale: 1.
	viewer calculateDataArray.
	self setVolume: snd loudness.
	volume setScaledValue: snd loudness.
	
	(graph data size > 5000)
		ifTrue: [deltaVal := 200]
		ifFalse: [deltaVal := (graph data size) // 10].

	self updateSlider! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: 'LY 7/25/2003 10:16'!
startSelection
		
	^ startSelection.! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: 'LY 7/25/2003 10:16'!
startSelection: aNumber


	startSelection := aNumber.! !

!ScratchSoundEditor methodsFor: 'graph ops' stamp: 'LY 8/2/2003 17:25'!
startTime
	
	| ss |
	"secs := cursor / samplingRate.
	hrs := secs // 3600.
	mins := (secs \\ 3600) // 60.
	secs := secs \\ 60 asFloat.
	ms := ((secs \\ 1) * 100) asFloat truncated.
	secs := secs // 1."

	" hrs asString,':',mins asString, ':', secs asString, '.', ms asString."
	
	origSamplingRate isNil
		ifTrue: [ss := 20000]
		ifFalse: [ss := origSamplingRate].
	^ ((cursor asFloat/ss) roundTo: 0.01) asString.! !

!ScratchSoundEditor methodsFor: 'graph ops' stamp: 'jm 2/3/2009 14:27'!
startTime: hrMinSecs
	
	| secs hrs mins hms |
	hms := hrMinSecs findTokens: ' :.'.
	hrs := hms at: 1.
	mins := hms at: 2.
	secs := hms at: 3.
	self startIndex: ((hrs asNumber * 3600) + (mins asNumber * 60) + secs asNumber) * samplingRate.
! !

!ScratchSoundEditor methodsFor: 'stepping' stamp: 'kfr 4/18/2014 00:47'!
step

	| played prev |
	(SoundPlayer isPlaying: snd) ifTrue: [
		played := ((snd millisecondsSinceStart) * snd samples size) / (1000.0 * snd duration).
		prev := (completeSnd samples size - snd samples size).
		playCursor := (played + prev truncated min: graph data size).
		self changed].

	time contents: self startTime.

	scrollDelta = 0 ifFalse: [
		graph startIndex: graph startIndex + (scrollDelta/graph scale).
		self updateSliderValue.].

	graph computeSlider.
	self fixSliderRange.
! !

!ScratchSoundEditor methodsFor: 'stepping' stamp: 'jm 9/29/2003 20:27'!
stepTime


	^ 150
! !

!ScratchSoundEditor methodsFor: 'menu' stamp: 'tpr 7/24/2013 16:16'!
stopProcess
"replaces #stop since that is now commonly used in Morph and confuses Scratch."
	snd pause.
	playCursor := nil.
	! !

!ScratchSoundEditor methodsFor: 'other' stamp: 'LY 7/1/2003 14:14'!
stretch: sampleArray by: stretchFactor
	"Return an array consisting of the given samples \stretched in time by the given factor."

	| out end incr i frac index |
	out := OrderedCollection new: (stretchFactor * sampleArray size) asInteger + 1.
	end := (sampleArray size - 1) asFloat.
	incr := 1.0 / stretchFactor.
	i := 1.0.
	[i < end] whileTrue: [
		frac := i fractionPart.
		index := i truncated.
		i := i + incr.
		out addLast:
			(((1.0 - frac) * (sampleArray at: index)) + (frac * (sampleArray at: index + 1))) rounded].
	^ out asArray
! !

!ScratchSoundEditor methodsFor: 'graph ops' stamp: ''!
undo


	| tmpSound tmpSel tmpCursor tmpScale |

	undoSound ifNil: [^self].

	tmpSound := SampledSound samples: graph data samplingRate: samplingRate.
	tmpSel := graph selection copy.
	tmpCursor := cursor.
	tmpScale := graph scale/ScratchGraphMorph MinScale.

	graph data: undoSound samples copy.
	viewer data: graph data.
	
	"scales the botton viewer so that the data fits on the entire screen." 
	(tmpSound samples size - graph data size) abs > 3
		ifTrue: [ScratchGraphMorph MinScale: ((viewer extent x/ graph data size) min: 1).
				viewer scale: 1.
				graph scale: undoScale.
				viewer startIndex: 1.
				graph computeSlider.
				self fixSliderRange.].

	graph calculateDataArray.
	viewer calculateDataArray.

	snd setSamples: graph data samplingRate: samplingRate.
	completeSnd setSamples: graph data samplingRate: samplingRate.
	self selection: undoSel copy.
	cursor := undoCursor.

	undoSound := tmpSound.
	undoSel := tmpSel.
	undoCursor := tmpCursor.
	undoScale := tmpScale.
! !

!ScratchSoundEditor methodsFor: 'other' stamp: ''!
updateSlider

	"this is the order that these methods have to be called in order to update slider!!!!!!"
	graph computeSlider.  
	self fixSliderRange.
	self updateSliderValue.! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: ''!
updateSliderValue

	slider setScaledValue: ((graph startIndex/graph data size min: slider maxVal) max: 0).




! !

!ScratchSoundEditor methodsFor: 'accessing' stamp: ''!
viewer

	^viewer.! !

!SoundLibraryTool methodsFor: '*ScratchSoundEditor' stamp: 'kfr 4/20/2014 18:35'!
openRecorder
	RecordingControls new openInWorld! !
